/********************************************************************
 *         This example code is from the book:
 *
 *           Motif Debuggin and Performance Tuning
 *            ISBN 0-13-147984-9
 *         by
 *           Douglas Young
 *           Prentice Hall, 1995
 *
 *         Copyright 1994 by Prentice Hall
 *         All Rights Reserved
 *
 *  Permission to use, copy, modify, and distribute this software for 
 *  any purpose except publication and without fee is hereby granted, provided 
 *  that the above copyright notice appear in all copies of the software.
 * *****************************************************************************/

/**********************************************************
 * chooseone.c: Allow the user to select from a list  
 *              of command-line arguments
 *              Modified to add a cancel button.
 **********************************************************/
#include <Xm/Xm.h>
#include <Xm/List.h>
#include <Xm/Form.h>
#include <Xm/PushB.h>
#include <stdio.h>
#include <stdlib.h>

static void BrowseCallback ( Widget    widget, 
                             XtPointer clientData, 
                             XtPointer callData );

static void CancelCallback ( Widget widget, 
                             XtPointer clientData, 
                             XtPointer callData );

const char usageString[] = 
              "Usage: chooseone <non-zero list of choices>";

void main ( int argc, char **argv )
{
    Widget       shell, list, form, cancel;
    int          i;
    XmString    *xmstr;
    XtAppContext app;

    shell = XtAppInitialize ( &app, "Chooseone",  NULL, 0, 
                              &argc, argv, NULL, NULL, 0 );

    if ( argc <= 1 )
    {
        fprintf ( stderr, usageString );
        exit ( -1 );
    }

   /*
    * Create a form widget to manage the list and button.
    */

    form = XtCreateManagedWidget ( "form", 
                                   xmFormWidgetClass,
                                   shell, NULL, 0 );
   /*
    * Convert all command-line arguments to an array of
    * type XmString, ignoring argv[0].
    */

    xmstr = ( XmString * ) XtMalloc ( sizeof ( XmString ) *
                                      argc - 1 );
    for ( i = 1; i < argc; i++ )
        xmstr [i - 1] = 
                XmStringCreateLtoR ( argv[i],
                                     XmFONTLIST_DEFAULT_TAG );
   /*
    * Create the list widget and register a browse callback.
    */

    list = XmCreateScrolledList ( form, "list", NULL, 0 );
    XtManageChild ( list );

   /*
    * Add a cancel button and a cancel callback.
    */

    cancel = XtCreateManagedWidget ( "cancel",
                                     xmPushButtonWidgetClass, 
                                     form, NULL, 0 );
    XtAddCallback ( cancel, XmNactivateCallback, 
                    CancelCallback, NULL );    

   /*
    * This setvalues call was already here. Just add the form
    * attachments to make the list span the top half of the
    * programs window.
    */

    XtVaSetValues ( list,
                    XmNitems,            xmstr,
                    XmNitemCount,        argc - 1,
                    XmNvisibleItemCount, 20,
                    XmNtopAttachment,    XmATTACH_FORM,
                    XmNbottomWidget,     cancel,
                    XmNbottomAttachment, XmATTACH_WIDGET,
                    XmNleftAttachment,   XmATTACH_FORM,
                    XmNrightAttachment,  XmATTACH_FORM,
                    NULL );
   /*
    * Attach the button to the lower right corner of th window
    */

    XtVaSetValues ( cancel,
                    XmNtopAttachment,    XmATTACH_NONE,
                    XmNbottomAttachment, XmATTACH_FORM,
                    XmNleftAttachment,   XmATTACH_NONE,
                    XmNrightAttachment,  XmATTACH_FORM,
                    NULL );    


    XtAddCallback ( list, XmNbrowseSelectionCallback, 
                    BrowseCallback, NULL );

    XtRealizeWidget ( shell );
    XtAppMainLoop ( app );
}

static void CancelCallback ( Widget    w, 
                             XtPointer clientData, 
                             XtPointer callData)
{
    printf ( "None" );
    exit( 0 );
}


static void BrowseCallback ( Widget    w, 
                             XtPointer clientData, 
                             XtPointer callData)
{
   XmListCallbackStruct *cbs =
                         ( XmListCallbackStruct * ) callData;
   Boolean  result;
   char    *text;

  /*
   * Retrieve the character data from the compound string
   */

   if ( ( result = XmStringGetLtoR ( cbs->item, 
                                     XmFONTLIST_DEFAULT_TAG,
                                     &text ) ) == TRUE )
   {
      /*
       * If some text was retrieved, print it. Normally, this  
       * data should be freed, but the program is going to 
       * exit, anyway.
       */

       printf ( "%s\n", text );
   }

   exit ( 0 );
}
